package eventbasedexcelextractor;

import java.util.ArrayList;
import java.util.List;

import org.apache.poi.hssf.eventusermodel.FormatTrackingHSSFListener;
import org.apache.poi.hssf.eventusermodel.HSSFListener;
import org.apache.poi.hssf.model.HSSFFormulaParser;
import org.apache.poi.hssf.record.BOFRecord;
import org.apache.poi.hssf.record.BoundSheetRecord;
import org.apache.poi.hssf.record.FormulaRecord;
import org.apache.poi.hssf.record.LabelRecord;
import org.apache.poi.hssf.record.LabelSSTRecord;
import org.apache.poi.hssf.record.NoteRecord;
import org.apache.poi.hssf.record.NumberRecord;
import org.apache.poi.hssf.record.Record;
import org.apache.poi.hssf.record.SSTRecord;
import org.apache.poi.hssf.record.StringRecord;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;

/**
 * TextHSSFListener
 */
public class TextHSSFListener implements HSSFListener {

	// true is sheet names should be included
	boolean includeSheetNames = true;

	// true if get the formula string, false if return result of formula
	boolean formulasNotResults = false;
	
	FormatTrackingHSSFListener formatTrackingHSSFListener;
	private SSTRecord sstRecord;

	private final List<String> sheetNameList;
	final StringBuffer sheetNameStringBuffer = new StringBuffer();
	private int sheetNumber = -1;
	private int rowNumber;

	private boolean outputNextStringValue = false;
	private int nextRow = -1;

	/**
	 * constructor 
	 * @param formulasNotResults
	 */
	public TextHSSFListener(boolean formulasNotResults,
			boolean includeSheetNames) {
		
		sheetNameList = new ArrayList<String>();
		
		this.formulasNotResults = formulasNotResults;
		this.includeSheetNames = includeSheetNames;	
	}

	@Override
	public void processRecord(Record record) {
		String thisText = null;
		int thisRow = -1;

		switch (record.getSid()) {
		
		case BoundSheetRecord.sid:
			BoundSheetRecord boundSheetRecord = (BoundSheetRecord) record;
			sheetNameList.add(boundSheetRecord.getSheetname());
			break;
			
		case BOFRecord.sid:
			BOFRecord bofRecord = (BOFRecord) record;
			if (bofRecord.getType() == BOFRecord.TYPE_WORKSHEET) {
				sheetNumber++;
				rowNumber = -1;

				if (includeSheetNames) {
					if (sheetNameStringBuffer.length() > 0) {
						sheetNameStringBuffer.append("\n");
					}
					sheetNameStringBuffer.append(sheetNameList.get(sheetNumber));
				}
			}
			break;
			
		case SSTRecord.sid:
			sstRecord = (SSTRecord) record;
			break;

		case FormulaRecord.sid:
			FormulaRecord formulaRecord = (FormulaRecord) record;
			thisRow = formulaRecord.getRow();

			if (formulasNotResults) {
				thisText = HSSFFormulaParser.toFormulaString((HSSFWorkbook) null, formulaRecord.getParsedExpression());
			} 
			else {
				if (formulaRecord.hasCachedResultString()) {
					// Formula result is a string
					// This is stored in the next record
					outputNextStringValue = true;
					nextRow = formulaRecord.getRow();
				} 
				else {
					thisText = formatTrackingHSSFListener.formatNumberDateCell(formulaRecord);
				}
			}
			break;
			
		case StringRecord.sid:
			if (outputNextStringValue) {
				// String for formula
				StringRecord stringRecord = (StringRecord) record;
				thisText = stringRecord.getString();
				thisRow = nextRow;
				outputNextStringValue = false;
			}
			break;
			
		case LabelRecord.sid:
			LabelRecord labelRecord = (LabelRecord) record;
			thisRow = labelRecord.getRow();
			thisText = labelRecord.getValue();
			break;
			
		case LabelSSTRecord.sid:
			LabelSSTRecord labelSSTRecord = (LabelSSTRecord) record;
			thisRow = labelSSTRecord.getRow();
			if (sstRecord == null) {
				throw new IllegalStateException("No Label SST record found");
			}
			thisText = sstRecord.getString(labelSSTRecord.getSSTIndex()).toString();
			break;
			
		case NoteRecord.sid:
			NoteRecord noteRecord = (NoteRecord) record;
			thisRow = noteRecord.getRow();
			// TODO: Find object to match noteRecord.getShapeId()
			break;
			
		case NumberRecord.sid:
			NumberRecord numberRecord = (NumberRecord) record;
			thisRow = numberRecord.getRow();
			thisText = formatTrackingHSSFListener.formatNumberDateCell(numberRecord);
			break;
			
		default:
			break;
		}

		if (thisText != null) {
			if (thisRow != rowNumber) {
				rowNumber = thisRow;
				if (sheetNameStringBuffer.length() > 0)
					sheetNameStringBuffer.append("\n");
			} else {
				sheetNameStringBuffer.append("\t");
			}
			sheetNameStringBuffer.append(thisText);
		}
	}
}